﻿
using System;
using System.Drawing;
using System.Windows.Forms;



namespace WindowsFormsApplication1
{

    public partial class MainWin : Form
    {
        public const int samples = 7;       // number of reference points
        public int order = 3;               // order of the polynomial
        public const int datapoints = 1000; // number of datapoints for the interpolation

        public TextBox[,] tbm;
        public TextBox[] tbs;
        public TextBox[] tbx;

        public double[,] m = new double[samples, samples];
        public double[] x = new double[samples];
        public double[] d = new double[samples];

        double[] yp = new double[datapoints];
        double[] xp = new double[datapoints];
        double[] y_k = new double[samples];
        double[] x_k = new double[samples];

        public MainWin()
        {
            int i;
            tbs = new TextBox[10];
            tbx = new TextBox[10];

            InitializeComponent();

            cbOrder.SelectedIndex = 2;

            tbs[0] = textBoxY1; tbs[1] = textBoxY2; tbs[2] = textBoxY3; tbs[3] = textBoxY4; tbs[4] = textBoxY5;
            tbs[5] = textBoxY6; tbs[6] = textBoxY7; tbs[7] = textBoxY8; tbs[8] = textBoxY9; tbs[9] = textBoxY10;

            tbx[0] = textBoxX1; tbx[1] = textBoxX2; tbx[2] = textBoxX3; tbx[3] = textBoxX4; tbx[4] = textBoxX5;
            tbx[5] = textBoxX6; tbx[6] = textBoxX7; tbx[7] = textBoxX8; tbx[8] = textBoxX9; tbx[9] = textBoxX10;
 
            // some sample values for y
             y_k[0] = 2.63;
             y_k[1] = 1.18;
             y_k[2] = 1.16;
             y_k[3] = 1.54;
             y_k[4] = 2.65;
             y_k[5] = 5.41;
             y_k[6] = 7.67;

             // some sample values for y
             x_k[0] = 0.04;
             x_k[1] = 0.32;
             x_k[2] = 0.51;
             x_k[3] = 0.73;
             x_k[4] = 1.03;
             x_k[5] = 1.42;
             x_k[6] = 1.6;

             if (samples < 10)
            {
                for (i = 0; i < samples; i++)
                {
                    tbs[i].Text = Convert.ToString(y_k[i]);
                    tbx[i].Text = Convert.ToString(x_k[i]);

                }
                for (i = samples; i < 10; i++)
                {
                    tbs[i].Visible = false;
                    tbx[i].Visible = false;
                }
            }
        }

        public void Interpolate(double[] x, int maxPoints, double maxTime)
        {
            int i, k;
            for (i = 0; i < maxPoints; i++)
            {
                yp[i] = 0;
                xp[i] = (double)(i) * maxTime / (double)maxPoints;
                for (k = 0; k < order; k++)
                {   
                    yp[i] = yp[i] + x[k] * Math.Pow( xp[i], (double)(k));
                }
            }
        }

        public void DrawGraph(int maxPoints, double minX, double maxX, bool doClear)
        {
            Point p1, p2;
            int j;
            double maxTime = -1.0;
            double maxVal = -1.0;
            double minVal = 1.0;
            double scalefactor;
            bool drawCeroline;
            for (j = 0; j < maxPoints - 1; j++)
            {
                if ((xp[j] >= minX) && (xp[j] <= maxX))
                {
                    if (maxTime < xp[j])
                        maxTime = xp[j];
                    if (maxVal < yp[j])
                        maxVal = yp[j];
                    if (minVal > yp[j])
                        minVal = yp[j];
                }
            }
            maxTime = maxTime * 1.05;
            if (minVal < 0)
            {
                if (maxVal > 0)
                {
                    if (maxVal > Math.Abs(minVal))
                        scalefactor = pGraph.Height / maxVal / 2.2;
                    else
                        scalefactor = pGraph.Height / Math.Abs(minVal) / 2.2;
                }
                else
                    scalefactor = 5.0;
                drawCeroline = true;
            }
            else
            {
                scalefactor = pGraph.Height / (maxVal) / 1.1;
                drawCeroline = false;
            }
            p1 = new Point();
            p2 = new Point();
            Graphics g = pGraph.CreateGraphics();
            if (doClear)
                g.Clear(Color.White);
            Pen bluePen = new Pen(Color.Blue, 2);
            Pen redPen = new Pen(Color.Red, 2);
            Pen blackPen = new Pen(Color.Black, 2);
            if (drawCeroline)
            {
                p1.X = 0;
                p1.Y = (pGraph.Height / 2);
                p2.X = pGraph.Width;
                p2.Y = (pGraph.Height / 2);
                g.DrawLine(blackPen, p1, p2);
            }
            bluePen.Width = 1;
            if (maxTime > 0) // draw interpolated graph
            {
                for (j = 0; j < maxPoints - 1; j++)
                {
                    if ((xp[j] >= minX) && (xp[j] <= maxX))
                    {
                        p1.X = Convert.ToInt32(xp[j] * pGraph.Width / (double)maxTime);
                        p2.X = Convert.ToInt32(xp[j + 1] * pGraph.Width / (double)maxTime);
                        if (drawCeroline)
                        {
                            p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                            p2.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                        }
                        else
                        {
                            p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                            p2.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                        }
                        if (p2.X > p1.X)
                            g.DrawLine(bluePen, p1, p2);
                    }
                }
                for (j = 0; j < samples; j++) // mark reference points
                {
                    p1.X = Convert.ToInt32(x_k[j] * pGraph.Width / (double)maxTime);
                    if (drawCeroline)
                        p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                    else
                        p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                    g.DrawEllipse(redPen, (int)(p1.X - 1.0), (int)(p1.Y - 1.0), 2, 2);
                }
            }
            g.Dispose();
        }


        private void button1_Click(object sender, EventArgs e)
        {
            int i, j, k;
            if (cbOrder.SelectedIndex >= 0)
                order = cbOrder.SelectedIndex + 1;
            double[,] a = new double[order, order];
            double[] y = new double[order];
            TGauss gauss = new TGauss(order, a, y);
            if (samples < 10)
            {
                for (i = 0; i < samples; i++)
                {
                    if (tbs[i].Text != "")
                    {
                        y_k[i] = Convert.ToDouble(tbs[i].Text);
                        x_k[i] = Convert.ToDouble(tbx[i].Text);
                    }
                    else
                        y_k[i] = 0.0;
                }
            }

            // fill matrix
            for (i = 0; i < samples; i++)
            {
                d[i] = y_k[i];
                for (k = 0; k < order; k++)
                {
                    m[i, k] = Math.Pow(x_k[i], k);
                }   
            }

            //calc a * transposed(a) and y * transposed(a)
            for (i = 0; i < order; i++)
            {
                y[i] = 0.0;
                for (k = 0; k < samples; k++)
                    y[i] = y[i] + m[k, i] * d[k];

                for (j = 0; j < order; j++)
                {
                    a[j,i] = 0.0;
                    for (k = 0; k < samples; k++)
                    {
                        a[j, i] = a[j, i] + m[k,j] * m[k, i];
                    }
                }
            }

            /*solve Matrix equation by Gauss*/
            if (gauss.Eliminate())
            {
                x = gauss.Solve();
                Interpolate(x, datapoints, x_k[samples - 1]);
                DrawGraph(datapoints, x_k[0], x_k[samples - 1], true);
            }
            else
                MessageBox.Show("Matrix calculattion", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        private void pGraph_Paint(object sender, PaintEventArgs e)
        {
            DrawGraph(datapoints, x_k[0], x_k[samples - 1], false);
        }  
    }
}
